$Title  Dynamic TD/BU hybrid model - MCP Code

set	taup		Time periods including post-terminal /2005*2101/
	tau(taup)	Time periods /2005*2100/
	tau0(taup)	First time period
	tauL(taup)	Last time period
	tauPT(taup)	Post-terminal time period;

tau0(tau)	= yes$(ord(tau) = 1);
tauL(tau)	= yes$(ord(tau) = card(tau));
tauPT(taup)	= yes$(ord(taup) = card(taup));

parameter	year(tau)	Year as a numeric value;
year(tau) = 2005 + (ord(tau)-1);

alias(tau,ttau);


*	Include generic model parameterization

$include data

*	Set up model along the steady-state growth path 

parameter 
	esubt		Intertemporal elasticity of substitution         /0.5/,
	r		Baseline interest rate				 /0.05/,
	gr		Baseline growth rate				 /0.02/,
	depr		Depreciation rate				 /0.07/,
	kd0(kg)		Capital demands
	ks0		Capital stock
	pref(taup)	Reference path price level,
	qref(taup)	Reference path quantity level,
	k0		Base year capital stock,
	i0		Base year investment,
	pk0		Benchmark capital price
	rk0		Benchmark rental price ;

*	Move capital rents of fossil fuel production into sector-specific rents
*	and assign sector-specific capital stocks
sam("rent",ff) = sam("rent",ff) + sam("capital",ff);
sam("capital",ff) = 0;		

kd0("y") = -sam("capital","y");
kd0(xt)  = -xtelec("capital",xt);
ks0	 = sum(kg,kd0(kg));

qref(taup) = power(1+gr,	ord(taup)-1);
pref(taup) = power(1/(1+r),	ord(taup)-1);

pk0     = 1+r;
rk0     = r + depr;

*	Assign initial capital stock and investment consistent with steady state growth

k0(kg)	= kd0(kg)/rk0;
i0(kg)	= k0(kg)*(gr+depr);


sam("y","ra") = sam("y","ra") + sum(kg,i0(kg));
c0 = (-sam("y","ra")-sam("ele","ra")-sam("oil","ra") - sam("gas","ra"));  
display sam,i0;


parameter	eq(ff,tau)	Fossil-fuel rents
		en(n,tau)	Electricity inputs;

eq(ff,tau) = -sam("rent",ff)*qref(tau);
en(n,tau)  = nrsupply(n)*qref(tau);


*	For steady-state replication check there is no supply limit on active technologies
limit(t,tau) =  1e6*sum(xt, xtelec("ele",xt));

*	Definitions for renewable quota scenario
parameter	t_shr(tau)	Target share for green quota;
t_shr(tau) = 0;

*	Benchmark carbon emissions
carblim(tau) = sum(ff, carbon(ff))*qref(tau);


*	MCP formulation of the model

positive variables
*	Activities
	y(tau)		Aggregate output
	ele(t,tau)	Production levels for electricity by technology
	x(ff,tau)	Fossil fuel supplies
	c(tau)		Aggregate consumption (utility) formation	
	i(kg,tau)	Investment
	k(kg,tau)	Capital stock
	g		Public sector output
	w		Intertemporal welfare
	lsp(tau)	Labor supply
	cls(tau)	Full consumption (including leisure)

*	Prices
	py(tau)		Price of aggregate output
	pele(tau)	Price of electricity
	pf(ff,tau)	Price of fossil fuels
	pl(tau)		Wage rate
	pls(tau)	Leisure rate
	rk(kg,tau)	Return to capital
	pk(kg,tau)	Price of malleable capital for X (and NT elec)
	pq(ff,tau)	Rent on fossil fuel resources
	pn(n,tau)	Rent on natural resources
	pc(tau)		Consumption (utility) price index	
	pkt		Terminal capital stock
	plim(t,tau)	Administratively imposed limit
	pcarb(tau)	Carbon tax	
	pg		Public sector demand
	pw		Intertemporal welfare price index
	pcls(tau)	Price index of full consumption (including leisure)

*	Income levels
	ra		Representative agent
	govt		Government;

variables

*	Auxiliary variables	
	psi(tau)	Subsidy associated with green quota
	phi_ls		Lump-sum recycling 	
	phi_tc		Consumption tax recycling	
	phi_tl		Labor tax recycling
	kt		Terminal capital stock ;


equations
*       Zero profit conditions for activities linked to activity levels
zprf_y(tau)     Zero profit condition for macro production sector
zprf_ele(t,tau) Zero profit condition for alternative electricity supply technologies
zprf_x(ff,tau)  Zero profit condition for fossil fuel supplies
zprf_c(tau)     Zero profit condition for consumption
zprf_i(kg,tau)  Zero profit condition for investment
zprf_k(kg,tau)  Zero profit condition for capital good format		
zprf_lsp(tau)   Zero profit condition for labor supply
zprf_g          Zero profit condition for public good formation
zprf_w		Zero profit condition for aggreate utility 
zprf_cls(tau)	Zero profig condition for full consumption good


*       Market clearance condiditions for goods linked to prices
mkt_py(tau)     Market clearance condition for macro production good
mkt_pele(tau)   Market clearance condition for electricity
mkt_pf(ff,tau)  Market clearance condition for fossil fuels
mkt_pl(tau)     Market clearance condition for labor
mkt_pls(tau)    Market clearance conditon for leisure
mkt_rk(kg,tau)  Market clearance condition for services to malleable capital
mkt_pk(kg,tau)  Market clearance condition for malleable capital
mkt_pq(ff,tau)  Market clearance condition for fossil fuel resources
mkt_pn(n,tau)   Market clearance condition for natural resources
mkt_pcarb(tau)  Market clearance condition for carbon
mkt_plim(t,tau) Market clearance condition for capacity on electricity expansion
mkt_pc(tau)     Market clearance condition for final consumption good
mkt_pkt		Market clearance for terminal capital stock
mkt_pg          Market clearance condition for public good
mkt_pw		Market clearance condition for aggregate utility good
mkt_pcls(tau)	Market clearance condition for full consumption good

*       Income balance for representative agent linked to income level
inc_ra          Budget constraint for representative household
inc_govt        Budget constraint for government        

*       Additional constraints
eqy_psi(tau)    Endogenous subsidy to achieve renewable energy quota
eqy_phi_ls      Equal yield constraint for lump-sum recycling
eqy_phi_tc      Equal yield constraint for consumption tax recycling
eqy_phi_tl	Equal yield constraint for labor tax recycling,
eqy_kt		Terminal constraint;


parameter 
theta_l_kl	    Cost share of labor in value added of Y production 
theta_kl	    Cost share of value added in Y production
theta_ele_e	    Cost share of electricity in energy composite of Y
theta_ff_nele(ff)   Cost share of fossil fuel in non-electric energy composite of Y
theta_r_ff(ff)      Cost share of fossil fuel resource in fossil fuel production
theta_l_ff(ff)      Cost share of labor in non-resource input of fossil fuel production
theta_y_ff(ff)      Cost share of Y in non-resource input of fossil fuel production
theta_ele_ff(ff)    Cost share of electricity in non-resource input of fossil fuel production 
theta_f_ff(f,ff)    Cost share of other fossil fuels in non-resource input of fossil fuel production
theta_ele_c         Cost share of electricity in oil-electricity composite of final consmption
theta_y_c	    Cost share of Y in final consumption
theta_ff_nele_c(ff) Cost share of fossilf fuel in non-electric energy composite of Y
theta_y_t(t)	    Cost share of Y in electricity production by technology t
theta_k_t(t)        Cost share of capital in electricity production by technology t
theta_l_t(t)        Cost share of capital in electricity production by technology t
theta_ff_t(ff,t)    Cost share of fossil fuel in electricity production by technology t
theta_ls_cls	    Cost share of labor supply in full consumption
theta_cls_w(tau)    Cost share of consumption in period tau in lifetime consumption;	


theta_l_kl            = (-sam("labor","y"))/(-sam("labor","y") + (-sam("capital","y")));
theta_kl	      = (-sam("labor","y") + (-sam("capital","y")))/sam("y","y");	
theta_ele_e           = (-sam("ele","y"))/((-sam("ele","y")) + sum(ff, (-sam(ff,"y"))));
theta_ff_nele(ff)     = (-sam(ff,"y"))/ sum(f, (-sam(f,"y")));
theta_r_ff(ff)        = (-sam("rent",ff))/sam(ff,ff); 
theta_l_ff(ff)        = (-sam("labor",ff))/(sam(ff,ff) - (-sam("rent",ff)));
theta_y_ff(ff)        = (-sam("y",ff))/(sam(ff,ff) - (-sam("rent",ff)));  
theta_ele_ff(ff)      = (-sam("ele",ff))/(sam(ff,ff) - (-sam("rent",ff)));
theta_y_c	      = -sam("y","ra")/c0;
theta_ele_c	      = (-sam("ele","ra"))/(-sam("ele","ra") + sum(ff, (-sam(ff,"ra")))); 
theta_ff_nele_c(ff)   = (-sam(ff,"ra"))/sum(f, (-sam(f,"ra")));
theta_y_t(t)$xt(t)    = (-xtelec("y",t)/xtelec("ele",t));
theta_k_t(t)$xt(t)    = (-(xtelec("capital",t)+kx(t))/xtelec("ele",t));
theta_l_t(t)$xt(t)    = (-xtelec("labor",t)/xtelec("ele",t)); 
theta_ff_t(ff,t)$xt(t)= (-xtelec(ff,t)/xtelec("ele",t));
theta_y_t(t)$nt(t)    = (-ntelec("y",t)/ntelec("ele",t));
theta_k_t(t)$nt(t)    = (-ntelec("capital",t)); 
theta_l_t(t)$nt(t)    = (-ntelec("labor",t)/ntelec("ele",t)); 
theta_ls_cls	      = (ld0/(ld0 + c0));
theta_cls_w(tau)      = (qref(tau)*(c0 + ld0)*pref(tau))/(sum(ttau,(qref(ttau)*(c0 + ld0)*pref(ttau)))); 
theta_f_ff(f,ff)$(not sameas(f,ff)) = (-sam(f,ff))/(sam(ff,ff) - (-sam("rent",ff)));

set y_kg(kg)  /Y/;
set map(t,kg) /coal.coal,gas.gas,oil.oil,nuclear.nuclear,hydro.hydro,wind.wind,solar.solar,biomass.biomass/;


*       Definition of zero profit conditions

zprf_y(tau).. 
	(theta_kl*(pl(tau)**(theta_l_kl)*rk("y",tau)**(1-theta_l_kl))**(1-esub)
	+(1- theta_kl)*(pele(tau)**(theta_ele_e)
	*(sum(ff, theta_ff_nele(ff)*pf(ff,tau)**(1-esub_nele))**(1/(1-esub_nele)))
	**(1-theta_ele_e))**(1-esub))**(1/(1-esub))
	=G= py(tau);
 
zprf_ele(t,tau)$limit(t,tau)..           
	theta_y_t(t)*py(tau) + theta_l_t(t)*pl(tau) 
	+((-xtelec("capital",t)/xtelec("ele",t))*rk(t,tau))$xt(t)
	+ ((-ntelec("capital",t))*rk(t,tau))$nt(t)
	+ sum(ff,theta_ff_t(ff,t)*pf(ff,tau)) 
	+ sum(n, (-xtelec(n,t))*pn(n,tau))$xt(t) + sum(n, (-ntelec(n,t))*pn(n,tau))$nt(t) 
	+  plim(t,tau)$limit(t,tau)
	=G= pele(tau)*(1+psi(tau)$quota(t));

zprf_x(ff,tau)..	
	(theta_r_ff(ff)*pq(ff,tau)**(1-esub_ff(ff)) + (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl(tau)  
	+ theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	+ ((carbon(ff)/sam(ff,ff)*pcarb(tau)))$(dd and sam(ff,ff)) 
        =G= pf(ff,tau);

zprf_c(tau)..     
	(theta_y_c*((py(tau)*(1+tc0*phi_tc$dd))/(1+tc0$dd))**(1-esubc) 
	+ (1-theta_y_c)*(theta_ele_c*pele(tau)**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(ff, pf(ff,tau)**theta_ff_nele_c(ff)))**(1-esubc_ele))
	**((1-esubc)/(1-esubc_ele)))**(1/(1-esubc)) 
        =G= pc(tau);

zprf_g$dd..     
	sum(tau, (g0*qref(tau))/(sum(ttau,pref(ttau)*qref(ttau)*g0))*py(tau)) 
	=G= pg;

zprf_lsp(tau)..
	pls(tau)*(1+(tl0*phi_tl)$dd)/(1+tl0$dd) 
	=G= pl(tau);

zprf_w..  
	sum(tau, theta_cls_w(tau)*(pcls(tau)/pref(tau))**(1-esubt))**(1/(1-esubt))
	=G= pw;

zprf_cls(tau).. 
	(theta_ls_cls*pls(tau)**(1-esubl)  + (1-theta_ls_cls)*pc(tau)**(1-esubl))**(1/(1-esubl))
	=G= pcls(tau);

zprf_i(kg,tau)$tau(tau)..	py(tau) =G= pk(kg,tau+1) + pkt$tauL(tau);

zprf_k(kg,tau)$tau(tau)..	pk(kg,tau) =G=  rk(kg,tau)*(r+depr) + pk(kg,tau+1)*(1-depr) + (pkt*(1-depr))$tauL(tau);	
      	

*       Definition of market clearance conditions
mkt_py(tau)..
        y(tau)*sam("y","y")  =G= 
        sum(xt, ele(xt,tau)*(-xtelec("y",xt)/xtelec("ele",xt))) 
        + sum(nt, ele(nt,tau)*(-ntelec("y",nt)))  
	+ sum(ff, (-sam("y",ff))*x(ff,tau)
	* (((theta_r_ff(ff)*pq(ff,tau)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl(tau)  
	 + theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	/(theta_l_ff(ff)*pl(tau)  + theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau))) )**esub_ff(ff)))
        + (-sam("y","ra")/(1+tc0$dd))*c(tau)*( (pc(tau)/(py(tau)*(1+(tc0*phi_tc)$dd)))*(1+tc0$dd))**esubc
	+ sum(kg, i(kg,tau))$tau(tau)
        + ((g0*qref(tau))*g)$dd;
                               
mkt_pele(tau)..
        sum(t, ele(t,tau))  =G= 
        (-sam("ele","ra"))*c(tau)*(pc(tau)/ (theta_ele_c*pele(tau)**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(ff, pf(ff,tau)**theta_ff_nele_c(ff)))**(1-esubc_ele))
	**(1/(1-esubc_ele)))**esubc* (((theta_ele_c*pele(tau)**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(ff, pf(ff,tau)**theta_ff_nele_c(ff)))**(1-esubc_ele))
	**(1/(1-esubc_ele)))/pele(tau) )**esubc_ele
	+ sum(ff, (-sam("ele",ff))*x(ff,tau)
	* (((theta_r_ff(ff)*pq(ff,tau)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl(tau)  
	 + theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	/(theta_l_ff(ff)*pl(tau)  + theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau))) )**esub_ff(ff)))
	+ (-sam("ele","y"))*y(tau)*(py(tau)/(pele(tau)**(theta_ele_e)
	*(sum(ff, theta_ff_nele(ff)*pf(ff,tau)**(1-esub_nele))**(1/(1-esub_nele)))
	**(1-theta_ele_e)))**esub*((pele(tau)**(theta_ele_e)
	*(sum(ff, theta_ff_nele(ff)*pf(ff,tau)**(1-esub_nele))**(1/(1-esub_nele)))
	**(1-theta_ele_e))/pele(tau));

alias(ff,fff);

mkt_pf(ff,tau)..
	sam(ff,ff)*x(ff,tau) =G= 
        sum(xt, (-xtelec(ff,xt)/xtelec("ele",xt))*ele(xt,tau))
        + {(-sam(ff,"ra"))*c(tau)*(pc(tau)/((theta_ele_c*pele(tau)**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(f, pf(f,tau)**theta_ff_nele_c(f)))
	**(1-esubc_ele))**(1/(1-esubc_ele))))**esubc
	*((theta_ele_c*pele(tau)**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(f, pf(f,tau)**theta_ff_nele_c(f)))
	**(1-esubc_ele))**(1/(1-esubc_ele))/prod(f, pf(f,tau)**theta_ff_nele_c(f)))**esubc_ele
	* prod(f, pf(f,tau)**theta_ff_nele_c(f))/pf(ff,tau)}$(-sam(ff,"ra"))

	+ (-sam(ff,"y"))*y(tau)*(py(tau)/(pele(tau)**(theta_ele_e)*(sum(f, theta_ff_nele(f)*pf(f,tau)
	**(1-esub_nele))**(1/(1-esub_nele)))**(1-theta_ele_e)))**esub
	*(pele(tau)**(theta_ele_e)*(sum(f, theta_ff_nele(f)*pf(f,tau)
	**(1-esub_nele))**(1/(1-esub_nele)))**(1-theta_ele_e))/(sum(f, theta_ff_nele(f)*pf(f,tau)
	**(1-esub_nele))**(1/(1-esub_nele)))
	*((sum(f, theta_ff_nele(f)*pf(f,tau)
	**(1-esub_nele))**(1/(1-esub_nele)))/pf(ff,tau))**esub_nele

	+ sum(f$(not sameas(ff,f)), (-sam(ff,f))*x(f,tau)
	* (((theta_r_ff(f)*pq(f,tau)**(1-esub_ff(f)) +  (1-theta_r_ff(f))*( theta_l_ff(f)*pl(tau)  
	+ theta_y_ff(f)*py(tau) + theta_ele_ff(f)*pele(tau) 
	+ sum(fff$(not sameas(fff,f)), theta_f_ff(fff,f)*pf(fff,tau)))**(1-esub_ff(f)))**(1/(1-esub_ff(f)))
	/(theta_l_ff(f)*pl(tau) + theta_y_ff(f)*py(tau) + theta_ele_ff(f)*pele(tau) 
	+ sum(fff$(not sameas(fff,f)), theta_f_ff(fff,f)*pf(fff,tau))) )**esub_ff(f))); 

mkt_pl(tau)..                
	sam("labor","ra")*lsp(tau) =G= 
	(-sam("labor","y"))*y(tau)*(py(tau)/(pl(tau)**(theta_l_kl)*rk("y",tau)**(1-theta_l_kl)))**esub
	*(pl(tau)**(theta_l_kl)*rk("y",tau)**(1-theta_l_kl))/pl(tau)
	+ sum(ff, (-sam("labor",ff))*x(ff,tau)
	* (((theta_r_ff(ff)*pq(ff,tau)**(1-esub_ff(ff)) + (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl(tau)  
	+ theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	/(theta_l_ff(ff)*pl(tau) + theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau))) )**esub_ff(ff)))
        + sum(t,theta_l_t(t)*ele(t,tau));                

mkt_rk(kg,tau)..
	k(kg,tau)*(r+depr)
	=G=
	((-sam("capital","y"))*y(tau)*(py(tau)/(pl(tau)**(theta_l_kl)*rk("Y",tau)**(1-theta_l_kl)))**esub
	*(pl(tau)**(theta_l_kl)*rk("Y",tau)**(1-theta_l_kl))/rk("Y",tau))$y_kg(kg)
        + sum(xt$map(xt,kg), kd0(xt)/xtelec("ele",xt)*ele(xt,tau))
	+ sum(nt$map(nt,kg),(-ntelec("capital",nt))*ele(nt,tau));                

mkt_pq(ff,tau)..            
	eq(ff,tau) =G= 
	(-sam("rent",ff))*x(ff,tau)* 
	((theta_r_ff(ff)*pq(ff,tau)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl(tau)  
	+ theta_y_ff(ff)*py(tau) + theta_ele_ff(ff)*pele(tau) 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f,tau)))**(1-esub_ff(ff)))
	**(1/(1-esub_ff(ff)))/pq(ff,tau))**esub_ff(ff);          

mkt_pn(n,tau)..
	en(n,tau) =G= sum(nt,(-ntelec(n,nt))*ele(nt,tau)) + sum(xt,(-xtelec(n,xt))*ele(xt,tau));;

mkt_plim(t,tau)$limit(t,tau).. 
	limit(t,tau) =G= ele(t,tau);

mkt_pcarb(tau)$dd..     
	carblim(tau) =G= sum(ff,carbon(ff)*x(ff,tau));  

mkt_pg$dd ..
	(sum(tau,pref(tau)*qref(tau)*g0))*g*pg =G= govt ; 

mkt_pw..
	(sum(tau,(qref(tau)*(c0 + ld0)*pref(tau))))*w*pw =G= ra; 	


mkt_pcls(tau).. 
	cls(tau)*(c0 + ld0) =G= ((c0+ld0)*qref(tau))*w*((pw/pcls(tau))*pref(tau))**esubt; 


mkt_pls(tau)..
	(qref(tau)*(sam("labor","ra")/(1+tl0$dd)+ld0)) =G= 
	(sam("labor","ra")/(1+tl0$dd))*lsp(tau)
	+ ld0*cls(tau)*(pcls(tau)/pls(tau))**esubl;


mkt_pc(tau) ..               
	c0*c(tau) =G= 
	 c0*cls(tau)*(pcls(tau)/pc(tau))**esubl;

mkt_pkt.. 
      sum(tau$tauL(tau), sum(kg, (1-depr)*k(kg,tau) + i(kg,tau))) =G= kt;

mkt_pk(kg,tau)..  
      k(kg,tau-1)*(1-depr) + i(kg,tau-1) + (max(k0(kg),1e-6))$tau0(tau) =G=  k(kg,tau);


*       Income definition for representative agent
inc_ra..
   	sum(tau$tau0(tau), sum(kg, pk(kg,tau)*(max(k0(kg),1e-6))))
   	+ sum(tau, pls(tau)*(qref(tau)*(sam("labor","ra")/(1+tl0$dd)+ld0))
   	+ sum(ff, pq(ff,tau)*eq(ff,tau))
   	+ sum(n, pn(n,tau)*en(n,tau))
   	+ sum(t$limit(t,tau), plim(t,tau)*limit(t,tau))	
   	- (pc(tau)*qref(tau)*phi_ls)$dd
   	- sum(t$quota(t), ele(t,tau)*psi(tau)*pele(tau))) - pkt*kt
	=G= ra;

*       Income definition for government
inc_govt$dd..   
	sum(tau, carblim(tau)*pcarb(tau) 
	+ pc(tau)*qref(tau)*phi_ls 
	+ py(tau)*phi_tc*tc0*(-sam("y","ra")/(1+tc0$dd))*c(tau)*( (pc(tau)/(py(tau)*(1+(tc0*phi_tc)$dd)))*(1+tc0$dd))**esubc
	+ pls(tau)*tl0*phi_tl*(sam("labor","ra")/(1+tl0$dd))*lsp(tau))
	=G= govt;

*       Endogenous subsidy to assure renewables quota
eqy_psi(tau)$card(quota)..   
	sum(t$res(t), ele(t,tau)) =G= t_shr(tau)*sum(t, ele(t,tau));

*       Endogenous equal yield constraints
eqy_phi_ls$dd..             
	g =G= 1;

eqy_phi_tc$dd..             
	g =G= 1;          

eqy_phi_tl$dd..             
	g =G= 1;          

eqy_kt..	sum(tauL(tau), sum(kg,I(kg,tau) - (1+gr)*I(kg,tau-1))) =e= 0;


*       Define MCP model
model mcp_dynamic / zprf_y.y, zprf_ele.ele, zprf_x.x, zprf_c.c, 
		   zprf_g.g, zprf_w.w, zprf_lsp.lsp,zprf_i.i, zprf_k.k, zprf_cls.cls,
		   mkt_py.py, mkt_pele.pele, mkt_pf.pf, mkt_pl.pl, mkt_pls.pls                     
		   mkt_rk.rk, mkt_pk.pk, mkt_pkt.pkt, mkt_pq.pq, mkt_pn.pn, mkt_pcarb.pcarb,
		   mkt_plim.plim, mkt_pc.pc, mkt_pg.pg, mkt_pw.pw, mkt_pcls.pcls, 
		   inc_ra.ra,inc_govt.govt,  
		   eqy_kt.kt, eqy_psi.psi, eqy_phi_ls.phi_ls, eqy_phi_tc.phi_tc, eqy_phi_tl.phi_tl  
                /;

*       Benchmark initialization

*       In the base year new-vintage technologies are inactive
*       and the prices of backstop natural resources are zero
*       Extant technologies with capacity limits are assumed to
*       operate at the upper bound with a zero shadow value in the
*       base year

y.l(tau)	= qref(tau);
ele.l(t,tau)	= qref(tau);
x.l(ff,tau)	= qref(tau);
c.l(tau)	= qref(tau);
lsp.l(tau)	= qref(tau);
cls.l(tau)	= qref(tau);
i.l(kg,tau)	= qref(tau)*i0(kg);
k.l(kg,tau)	= qref(tau)*k0(kg);
w.l		= 1;
ele.l(xt,tau)   = xtelec("ele",xt) * qref(tau);
ele.l(nt,tau)   = 0;


pcls.l(tau)	= pref(tau);
pele.l(tau)	= pref(tau);
py.l(tau)	= pref(tau);
pf.l(ff,tau)	= pref(tau);
pl.l(tau)	= pref(tau);
pls.l(tau)	= pref(tau);
rk.l(kg,tau)	= pref(tau);
pk.l(kg,tau)	= pk0 * pref(tau);
pkt.l		= sum(tauL, pref(tauL));
pq.l(ff,tau)	= pref(tau);
pc.l(tau)	= pref(tau);
pw.l		= 1;
pn.l(n,tau)     = 0;
plim.l(t,tau)	= 0;
pcarb.l(tau)    = 0;
	
kt.l = sum(tauL(taup), qref(taup+1)) * sum(kg,k0(kg));

*	Assign/fix additional variables
psi.fx(tau)$(not card(quota)) = 0; plim.fx(t,tau)$(not limit(t,tau)) = 0;
phi_tc.fx = 1; phi_ls.fx = -tl0*sam("labor","ra")/(1+tl0); phi_tl.fx = 1; g.fx = 0; pg.fx = 0; govt.fx = 0; pcarb.fx(tau) = 0;


*	Provisions fo public sector extension 
if(dd,
  g0  = 0.15 *(-sam("y","ra"));
  tc0 = g0/((-sam("y","ra")) - g0);
  phi_ls.lo	= -inf; phi_ls.up = +inf; 
  phi_ls.l	= -tl0*sam("labor","ra")/(1+tl0); 
  phi_tc.fx   	= 1;
  phi_tl.fx   	= 1; 
  pg.lo	= 0; pg.up = +inf; pg.l = 1;
  g.lo	= 0; g.up = +inf; g.l = 1;
  govt.lo = 0; govt.up = +inf; 
  pcarb.lo(tau) = 0; pcarb.up(tau) = +inf; pcarb.l(tau) = 0;
  );


ra.l   =sum(tau$tau0(tau), sum(kg, pk.l(kg,tau)*(max(k0(kg),1e-6))))
	+ sum(tau, pls.l(tau)*(qref(tau)*(sam("labor","ra")/(1+tl0$dd)+ld0))
	+ sum(ff, pq.l(ff,tau)*eq(ff,tau))
	+ sum(n, pn.l(n,tau)*en(n,tau))
	+ sum(t$limit(t,tau), plim.l(t,tau)*limit(t,tau))	
	- (pc.l(tau)*qref(tau)*phi_ls.l)$dd
	- sum(t$quota(t), ele.l(t,tau)*psi.l(tau)*pele.l(tau)))
	- pkt.l*kt.l;

govt.l$dd = sum(tau, carblim(tau)*pcarb.l(tau) 
	+ pc.l(tau)*qref(tau)*phi_ls.l 
	+ py.l(tau)*phi_tc.l*tc0*(-sam("y","ra")/(1+tc0$dd))*c.l(tau)*( (pc.l(tau)/(py.l(tau)*(1+(tc0*phi_tc.l)$dd)))*(1+tc0$dd))**esubc
	+ pls.l(tau)*tl0*phi_tl.l*(sam("labor","ra")/(1+tl0$dd))*lsp.l(tau));

mcp_dynamic.iterlim = 0;
solve mcp_dynamic using mcp;


display "The precision of the benchmark dataset is:", mcp_dynamic.objval;
abort$(ABS(mcp_dynamic.objval) gt 1e-4)"MCP model does not calibrate"; 

*	Relax iteration limit for counterfactual policy analysis

mcp_dynamic.iterlim      = 40000;

*	Fix a numeraire
pw.fx = 1;


*	First compute off-the-steady-state baseline accounting for resource
*	limits on renewables and policy constraint on nuclear
*	- Hydro production is fixed at the baseyear level
*	- Other natural resource capacities are given in the base year as a fraction 
*	  of electricity output and thereafter can increase at the square of the 
*	  the economic growth rate up to a maximum of 30 % of reference (steady-state) 
*	  electricity production
*	- Nuclear power can not exceed the base-year level

en(n,tau)		= nrsupply(n) * min(qref(tau)**2, 0.3*qref(tau)*sam("ele","ele"));
en("water", tau) 	= nrsupply("water");
limit("nuclear",tau)	= xtelec("ele","nuclear");

solve mcp_dynamic using mcp;

*	Store BaU values

parameter	bau_stat	BaU statistics;

bau_stat(tau,t)		= ele.l(t,tau);
bau_stat(tau,"res")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));
bau_stat(tau,"carbon")	= sum(ff, x.l(ff,tau)*carbon(ff));
bau_stat(tau,"c")	= c.l(tau);
bau_stat("XXX","w")	= w.l;
bau_stat(tau,"i")	= sum(kg,i.l(kg,tau));
bau_stat(tau,"ele")	= sum(t,ele.l(t,tau));
bau_stat(tau,"y")	= y.l(tau);
bau_stat(tau,"rq")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));


*===========================================================================
*	Dynamic analysis of policy scenarios (as laid out in the paper)
*
*	Scenario 1:	Gradual nuclear phase-out
*	Scenario 2:	Target quota for renewables (green quota)
*	Scenario 3:	Carbon taxation (environmental tax reform)

*	Across all scenarios we assume that the policy regulation will take
*	place in the time interval given by [taustart, tauend]

scalar	taustart	Start year of regulatory policy change	/2010/,
	tauend		End year of regulatory policy change	/2030/;

set taulbl(tau)/2010,2020,2030,2040,2050,2060,2070,2080,2090,2100/;

$setglobal labels taulbl
$setglobal domain tau
$setglobal gp_opt1 "set key below"
$setglobal gp_opt2 "set xlabel 'Time'"

parameter	delta		Changes in macroeconomic parameters (in percent),
		subsidy		Subsidy rate on electricity from renewables,		
		carbtax		Carbon permit price (carbon tax),
		rep_w		Percentage change in welfare (HEV);

*===========================================================================
*	Scenario 1: Gradual nuclear phase-out

limit("nuclear",tau) = 	bau_stat(tau,"nuclear");

loop(tau$(year(tau) GE taustart),
	limit("nuclear",tau)	= max(0, sum(ttau$(year(ttau) eq taustart), 
				     bau_stat(ttau,"nuclear"))*(1- (year(tau) - taustart)/(tauend-taustart)));
*       As nuclear capacity goes down to zero, shut down electricity production from nuclear
	ele.fx("nuclear",tau)$(not limit("nuclear",tau))	= 0;
);

solve mcp_dynamic using mcp;

delta(tau,"c")    = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
delta(tau,"i")    = 100 * (sum(kg,i.l(kg,tau))/bau_stat(tau,"i")-1);
delta(tau,"ele")  = 100 * (sum(t,ele.l(t,tau))/bau_stat(tau,"ele")-1);
rep_w("nuclear","XXX")  = round(100*(w.l/bau_stat("XXX","w") - 1),4);	 

$setglobal gp_opt3 "set ylabel '% change'"
$libinclude plot delta
delta(tau,"c") = 0; delta(tau,"i") = 0; delta(tau,"ele") = 0;

*	Re-initialize parameterization for subsequent scenarios
limit("nuclear",tau) = xtelec("ele","nuclear");
ele.lo("nuclear",tau) = 0; ele.up("nuclear",tau) = +inf;

*===========================================================================
*	Scenario 2: Green quotas
*
*	The share of power from renewable technologies in overall power production
*	is increased linearily by 10% relative to 2005 from "taustart" until "tauend" 
*	and then kept constant
t_shr(tau)			   = bau_stat("2005","rq");
t_shr(tau)$(year(tau) ge taustart) = min(1, sum(ttau$(year(ttau) eq taustart), 
					bau_stat("2005","rq")) + 10/100*(year(tau) - taustart)/(tauend-taustart));
t_shr(tau)$(year(tau) ge tauend)   = sum(ttau$(year(ttau) eq tauend), t_shr(ttau)); 


*	Impose green quota
quota(t)$res(t) = yes;
psi.lo(tau) = -inf; psi.up(tau) = +inf; psi.l(tau) = 0;  

solve mcp_dynamic using mcp;

delta(tau,"c")    = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
delta(tau,"i")    = 100 * (sum(kg,i.l(kg,tau))/bau_stat(tau,"i")-1);
delta(tau,"ele")  = 100 * (sum(t,ele.l(t,tau))/bau_stat(tau,"ele")-1);
subsidy(tau)$(year(tau) ge taustart) = eps + 100*psi.l(tau);
rep_w("quota","XXX")  = round(100*(w.l/bau_stat("XXX","w") - 1),4);	 

$setglobal gp_opt3 "set ylabel '% change'"
$libinclude plot delta

$setglobal gp_opt3 "set ylabel 'Subsidy rate (% of electricity price)'"
$libinclude plot subsidy

display rep_w;
delta(tau,"c") = 0; delta(tau,"i") = 0; delta(tau,"ele") = 0;
subsidy(tau) = 0;

*	Re-initialize parameterization for subsequent scenarios
quota(res) = no; psi.fx(tau)$(not card(quota)) = 0;


*===========================================================================
*	Scenario 3: Carbon taxation (Environmental tax reform)
*	Linear reduction of carbon emissions versus BaU by 20%

*	First re-calibrate base year (benchmark) accounting for public good extension
dd  = 1;	
g0  = 0.15 *(-sam("y","ra"));
tc0 = g0/((-sam("y","ra")) - g0);
phi_ls.lo = -inf; phi_ls.up = +inf; phi_ls.l = -tl0*sam("labor","ra")/(1+tl0); 
phi_tc.fx$dd = 1; phi_tl.fx$dd	= 1; 
pg.lo	= 0; pg.up   = +inf; pg.l = 1;
g.lo	= 0; g.up    = +inf; g.l = 1;
govt.lo	= 0; govt.up = +inf; 

*	Compute new BaU with public good provision and initial technology constraints
solve mcp_dynamic using mcp;

*	Store BaU values
bau_stat(tau,t)		= ele.l(t,tau);
bau_stat(tau,"res")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));
bau_stat(tau,"carbon")	= sum(ff, x.l(ff,tau)*carbon(ff));
bau_stat(tau,"c")	= c.l(tau);
bau_stat("XXX","w")	= w.l;
bau_stat(tau,"i")	= sum(kg,i.l(kg,tau));
bau_stat(tau,"ele")	= sum(t,ele.l(t,tau));
bau_stat(tau,"y")	= y.l(tau);
bau_stat(tau,"rq")	= sum(t$res(t), ele.l(t,tau))/sum(t, ele.l(t,tau));
bau_stat("xxx","ls")	= phi_ls.l;
display bau_stat;


*	Linear reduction of carbon emissions by 20% between 
*	"taustart" and "tauend" holding vis-a-vis BaU values 

pcarb.lo(tau) = 0; pcarb.up(tau) = +inf; pcarb.l(tau) = 0;
carblim(tau)	=  bau_stat(tau,"carbon");
carblim(tau)$((year(tau) ge taustart) and (year(tau) le tauend))
		=  bau_stat(tau,"carbon")*(1- 0.2*(year(tau) - taustart)/(tauend-taustart));
carblim(tau)$(year(tau) ge tauend)	=  (1-0.2)*bau_stat(tau,"carbon"); 


parameter	cdelta	Change in consumption (in percent);

*	Activate lump-sum transfer as recycling instrument
solve mcp_dynamic using mcp;

delta(tau,"c")    = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
delta(tau,"i")    = 100 * (sum(kg,i.l(kg,tau))/bau_stat(tau,"i")-1);
delta(tau,"ele")  = 100 * (sum(t,ele.l(t,tau))/bau_stat(tau,"ele")-1);
carbtax(tau,"ls") = round((1/multi)*pcarb.l(tau)/pc.l(tau), 1);
rep_w("ctax","ls")= round(100*(w.l/bau_stat("XXX","w") - 1),4);	 
cdelta(tau,"ls")  = 100 * (c.l(tau)/bau_stat(tau,"c")-1);

*	Activate consumption tax as recycling instrument
	phi_tc.l = 1; phi_tc.lo = -0.99;  phi_tc.up = +inf; phi_ls.fx = bau_stat("XXX","ls"); phi_tl.fx = 1;
solve mcp_dynamic using mcp;

cdelta(tau,"tc")  = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
carbtax(tau,"tc") = round((1/multi)*pcarb.l(tau)/pc.l(tau), 1);
rep_w("ctax","tc")= round(100*(w.l/bau_stat("XXX","w") - 1),4);	 


*	Activate wage tax as recycling instrument
	phi_tl.l = 1; phi_tl.lo = -0.99; phi_tl.up = +inf; phi_tc.fx = 1; phi_ls.fx = bau_stat("XXX","ls");
solve mcp_dynamic using mcp;

cdelta(tau,"tl")  = 100 * (c.l(tau)/bau_stat(tau,"c")-1);
carbtax(tau,"tl") = round((1/multi)*pcarb.l(tau)/pc.l(tau), 1);
rep_w("ctax","tl")= round(100*(w.l/bau_stat("XXX","w") - 1),4);	 

$setglobal gp_opt3 "set ylabel '% change'"
$libinclude plot cdelta

$setglobal gp_opt3 "set ylabel 'Carbon tax'"
$libinclude plot carbtax

display rep_w;
