$Title  Static TD/BU hybrid model - MCP Code

*	Include generic model parameterization
$include data

*	Initially there is only a policy constraint on the use of nuclear power
*	which is fixed at the base year level (operating at a shadow price of zero)
limit(t) = 1e+6 * elecgen("total");
limit("nuclear")	= xtelec("ele","nuclear");

set	quota(t)  Flag for technologies contributing to green quota;
scalar	t_shr	  Target share for green quota;

quota(t)= no; t_shr	= 0;

*	Benchmark carbon emissions
carblim = sum(ff, carbon(ff));


*	MCP formulation of the model
positive variables
*       Activitiy levels
	y	Aggregate output
	ele(t)	Production levels for electricity by technology
	x(ff)	Fossil fuel supplies
	c	Aggregate final consumption
	lsp	Labor supply
	g   	Public good provision
	w	Full consumption (utility) including leisure	

*       Price levels
	py	Price of aggregate output
	pele	Price of electricity
	pf(ff)	Price of fossil fuels
	pl	Wage rate
	pls	Leisure rate
	pk	Price of malleable capital for X (and NT elec)
	pq(ff)	Rent on fossil fuel resources
	pn(n)	Rent on natural resources
	pc	Consumption (utility) price index	
	pw	Price of full consumption including leisure
	plim(t)	Shadow price on electricity expansion
	pkx(t)	Price of capital to extant technologies
	pg	Price of public consumption
	pcarb	Carbon tax rate
	
*       Income variables
	ra	Representative agent
	govt	Government;


*       Endogenous taxes or subsidies
variables
	psi	Uniform subsidy rate on renewable energy	
	phi_ls	Lump-sum recycling of carbon tax revenues 	
	phi_tc	Consumption tax recycling of carbon tax revenues	
	phi_tl	Labor tax recycle of carbon tax revenue;


equations
*       Zero profit conditions for activities linked to activity levels
zprf_y          Zero profit condition for macro production sector
zprf_ele(t)     Zero profit condition for alternative electricity supply technologies
zprf_x(ff)      Zero profit condition for fossil fuel supplies
zprf_c          Zero profit condition for consumption
zprf_g          Zero profit condition for public good formation
zprf_lsp	Zero profit condition for labor supply
zprf_w		Zero profit condition for aggreate utility (full consumption)

*       Market clearance condiditions for goods linked to prices
mkt_py	        Market clearance condition for macro production good
mkt_pele        Market clearance condition for electricity
mkt_pf(ff)      Market clearance condition for fossil fuels
mkt_pl          Market clearance condition for labor
mkt_pls		Market clearance conditon for leisure
mkt_pk          Market clearance condition for malleable capital
mkt_pq(ff)      Market clearance condition for fossil fuel resources
mkt_pn(n)       Market clearance condition for natural resources
mkt_pcarb       Market clearance condition for carbon
mkt_pkx(t)      Market clearance condition for capital inputs to extant power production
mkt_plim(t)     Market clearance condition for capacity on electricity expansion
mkt_pc          Market clearance condition for final consumption good
mkt_pg          Market clearance condition for public good
mkt_pw		Market clearance condition for aggregate utility good

*       Income balance for representative agent linked to income level
inc_ra          Budget constraint for representative household
inc_govt        Budget constraint for government        


*       Additional constraints
eqy_psi         Endogenous subsidy to achieve renewable energy quota
eqy_phi_ls      Equal yield constraint for lump-sum recycling
eqy_phi_tc      Equal yield constraint for consumption tax recycling
eqy_phi_tl	Equal yield constraint for labor tax recycling;

parameter 
theta_l_kl	    Cost share of labor in value added of Y production 
theta_kl	    Cost share of value added in Y production
theta_ele_e	    Cost share of electricity in energy composite of Y
theta_ff_nele(ff)   Cost share of fossil fuel in non-electric energy composite of Y
theta_r_ff(ff)      Cost share of fossil fuel resource in fossil fuel production
theta_l_ff(ff)      Cost share of labor in non-resource input of fossil fuel production
theta_k_ff(ff)      Cost share of capital in non-resource input of fossil fuel production
theta_y_ff(ff)      Cost share of Y in non-resource input of fossil fuel production
theta_ele_ff(ff)    Cost share of electricity in non-resource input of fossil fuel production 
theta_f_ff(f,ff)    Cost share of other fossil fuels in non-resource input of fossil fuel production
theta_ele_c         Cost share of electricity in oil-electricity composite of final consmption
theta_y_c	    Cost share of Y in final consumption
theta_ff_nele_c(ff) Cost share of fossilf fuel in non-electric energy composite of Y
theta_y_t(t)	    Cost share of Y in electricity production by technology t
theta_k_t(t)        Cost share of capital in electricity production by technology t
theta_l_t(t)        Cost share of capital in electricity production by technology t
theta_ff_t(ff,t)    Cost share of fossil fuel in electricity production by technology t
theta_ls	    Cost share of labor supply in aggregate utility composite; 

theta_l_kl        = (-sam("labor","y"))/(-sam("labor","y") + (-sam("capital","y")));
theta_kl          = (-sam("labor","y") + (-sam("capital","y")))/sam("y","y");	
theta_ele_e       = (-sam("ele","y"))/((-sam("ele","y")) + sum(ff, (-sam(ff,"y"))));
theta_ff_nele(ff) = (-sam(ff,"y"))/ sum(f, (-sam(f,"y")));
theta_r_ff(ff)    = (-sam("rent",ff))/sam(ff,ff); 
theta_l_ff(ff)    = (-sam("labor",ff))/(sam(ff,ff) - (-sam("rent",ff)));
theta_k_ff(ff)    = (-sam("capital",ff))/(sam(ff,ff) - (-sam("rent",ff)));
theta_y_ff(ff)    = (-sam("y",ff))/(sam(ff,ff) - (-sam("rent",ff)));  
theta_ele_ff(ff)  = (-sam("ele",ff))/(sam(ff,ff) - (-sam("rent",ff)));
theta_f_ff(f,ff)$(not sameas(f,ff)) = (-sam(f,ff))/(sam(ff,ff) - (-sam("rent",ff)));
theta_y_c	  = -sam("y","ra")/c0;
theta_ele_c	  = (-sam("ele","ra"))/(-sam("ele","ra") + sum(ff, (-sam(ff,"ra")))); 
theta_ff_nele_c(ff)   = (-sam(ff,"ra"))/sum(f, (-sam(f,"ra")));
theta_y_t(t)$xt(t)    = (-xtelec("y",t)/xtelec("ele",t));
theta_k_t(t)$xt(t)    = (-(xtelec("capital",t)+kx(t))/xtelec("ele",t));
theta_l_t(t)$xt(t)    = (-xtelec("labor",t)/xtelec("ele",t)); 
theta_ff_t(ff,t)$xt(t)= (-xtelec(ff,t)/xtelec("ele",t));
theta_y_t(t)$nt(t)    = (-ntelec("y",t)/ntelec("ele",t));
theta_k_t(t)$nt(t)    = (-ntelec("capital",t)); 
theta_l_t(t)$nt(t)    = (-ntelec("labor",t)/ntelec("ele",t)); 
theta_ls	      = ld0/(c0 + ld0);	

*       Definition of zero profit conditions
zprf_y.. 
	(theta_kl*(pl**(theta_l_kl)*pk**(1-theta_l_kl))**(1-esub)
	+(1- theta_kl)*(pele**(theta_ele_e)
	*(sum(ff, theta_ff_nele(ff)*pf(ff)**(1-esub_nele))**(1/(1-esub_nele)))
	**(1-theta_ele_e))**(1-esub))**(1/(1-esub))
	=G= py;
 
zprf_ele(t)..           
	theta_y_t(t)*py + theta_l_t(t)*pl 
	+((-(xtelec("capital",t)+kx(t))/xtelec("ele",t))*pk)$xt(t)
	+ ((kx(t)/xtelec("ele",t))*pkx(t))$xt(t)
	+ ((-ntelec("capital",t))*pk)$nt(t)
	+ {sum(ff,theta_ff_t(ff,t)*pf(ff)) + sum(n, (-xtelec(n,t))*pn(n)) +  plim(t)$limit(t)}$xt(t)
	+ sum(n, (-ntelec(n,t))*pn(n))$nt(t)
	=G= pele*(1+psi$quota(t));

zprf_x(ff)..	
	(theta_r_ff(ff)*pq(ff)**(1-esub_ff(ff)) + (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl  
	+ theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	+ ((carbon(ff)/sam(ff,ff)*pcarb))$(carblim and sam(ff,ff)) 
        =G= pf(ff);

zprf_c..     
	(theta_y_c*((py*(1+tc0*phi_tc$dd))/(1+tc0$dd))**(1-esubc) 
	+ (1-theta_y_c)*(theta_ele_c*pele**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(ff, pf(ff)**theta_ff_nele_c(ff)))**(1-esubc_ele))
	**((1-esubc)/(1-esubc_ele)))**(1/(1-esubc)) 
        =G= pc;

zprf_g$dd..     
	py 
	=G= pg;

zprf_lsp..
	pls*(1+tl0*phi_tl$dd)/(1+tl0$dd) 
	=G= pl;

zprf_w..
	(theta_ls*pls**(1-esubl) + (1-theta_ls)*pc**(1-esubl))**(1/(1-esubl))
	=G= pw;


*       Definition of market clearance conditions
mkt_py..
        y*sam("y","y")  =G= 
        sum(xt, ele(xt)*(-xtelec("y",xt)/xtelec("ele",xt))) 
        + sum(nt, ele(nt)*(-ntelec("y",nt)))  
	+ sum(ff, (-sam("y",ff))*x(ff)
	* (((theta_r_ff(ff)*pq(ff)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl  
	+ theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	/(theta_l_ff(ff)*pl + theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f))) )**esub_ff(ff)))
        + (-sam("y","ra")/(1+tc0$dd))*c*( (pc/(py*(1+(tc0*phi_tc)$dd)))*(1+tc0$dd))**esubc
        + (g0*g)$dd;
                                  
mkt_pele..
        sum(t, ele(t))  =G= 
        (-sam("ele","ra"))*c*(pc/ (theta_ele_c*pele**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(ff, pf(ff)**theta_ff_nele_c(ff)))**(1-esubc_ele))
	**(1/(1-esubc_ele)))**esubc* (((theta_ele_c*pele**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(ff, pf(ff)**theta_ff_nele_c(ff)))**(1-esubc_ele))
	**(1/(1-esubc_ele)))/pele )**esubc_ele
	+ sum(ff, (-sam("ele",ff))*x(ff)
	* (((theta_r_ff(ff)*pq(ff)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl  
	+ theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	/(theta_l_ff(ff)*pl + theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f))) )**esub_ff(ff)))
	+ (-sam("ele","y"))*y*(py/(pele**(theta_ele_e)
	*(sum(ff, theta_ff_nele(ff)*pf(ff)**(1-esub_nele))**(1/(1-esub_nele)))
	**(1-theta_ele_e)))**esub*((pele**(theta_ele_e)
	*(sum(ff, theta_ff_nele(ff)*pf(ff)**(1-esub_nele))**(1/(1-esub_nele)))
	**(1-theta_ele_e))/pele);

alias(ff,fff);

mkt_pf(ff)..
	sam(ff,ff)*x(ff) =G= 
        sum(xt, (-xtelec(ff,xt)/xtelec("ele",xt))*ele(xt))

        + {(-sam(ff,"ra"))*c*(pc/((theta_ele_c*pele**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(f, pf(f)**theta_ff_nele_c(f)))
	**(1-esubc_ele))**(1/(1-esubc_ele))))**esubc
	*((theta_ele_c*pele**(1-esubc_ele) 
	+(1-theta_ele_c)*(prod(f, pf(f)**theta_ff_nele_c(f)))
	**(1-esubc_ele))**(1/(1-esubc_ele))/prod(f, pf(f)**theta_ff_nele_c(f)))**esubc_ele
	* prod(f, pf(f)**theta_ff_nele_c(f))/pf(ff)}$(-sam(ff,"ra"))
	+ (-sam(ff,"y"))*y*(py/(pele**(theta_ele_e)*(sum(f, theta_ff_nele(f)*pf(f)
	**(1-esub_nele))**(1/(1-esub_nele)))**(1-theta_ele_e)))**esub
	*(pele**(theta_ele_e)*(sum(f, theta_ff_nele(f)*pf(f)
	**(1-esub_nele))**(1/(1-esub_nele)))**(1-theta_ele_e))/(sum(f, theta_ff_nele(f)*pf(f)
	**(1-esub_nele))**(1/(1-esub_nele)))
	*((sum(f, theta_ff_nele(f)*pf(f)
	**(1-esub_nele))**(1/(1-esub_nele)))/pf(ff))**esub_nele
	+ sum(f$(not sameas(ff,f)), (-sam(ff,f))*x(f)
	* (((theta_r_ff(f)*pq(f)**(1-esub_ff(f)) +  (1-theta_r_ff(f))*( theta_l_ff(f)*pl  
	+ theta_k_ff(f)*pk + theta_y_ff(f)*py + theta_ele_ff(f)*pele 
	+ sum(fff$(not sameas(fff,f)), theta_f_ff(fff,f)*pf(fff)))**(1-esub_ff(f)))**(1/(1-esub_ff(f)))
	/(theta_l_ff(f)*pl + theta_k_ff(f)*pk + theta_y_ff(f)*py + theta_ele_ff(f)*pele 
	+ sum(fff$(not sameas(fff,f)), theta_f_ff(fff,f)*pf(fff))) )**esub_ff(f))); 

mkt_pl..                
	sam("labor","ra")*lsp =G= 
	(-sam("labor","y"))*y*(py/(pl**(theta_l_kl)*pk**(1-theta_l_kl)))**esub
	*(pl**(theta_l_kl)*pk**(1-theta_l_kl))/pl
	+ sum(ff, (-sam("labor",ff))*x(ff)
	* (((theta_r_ff(ff)*pq(ff)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl  
	+ theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	/(theta_l_ff(ff)*pl + theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f))) )**esub_ff(ff)))
        + sum(t,theta_l_t(t)*ele(t));                

mkt_pk..
	(-sam("capital","y")+ sum(ff,-sam("capital",ff)) + sum(xt,(-xtelec("capital",xt)) - kx(xt))) =G=
	(-sam("capital","y"))*y*(py/(pl**(theta_l_kl)*pk**(1-theta_l_kl)))**esub
	*(pl**(theta_l_kl)*pk**(1-theta_l_kl))/pk
	+ sum(ff, (-sam("capital",ff))*x(ff)
	*(((theta_r_ff(ff)*pq(ff)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl  
	+ theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f)))**(1-esub_ff(ff)))**(1/(1-esub_ff(ff)))
	/(theta_l_ff(ff)*pl + theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f))) )**esub_ff(ff)))
	+ sum(xt, (-(xtelec("capital",xt)+kx(xt))/xtelec("ele",xt))*ele(xt))
	+ sum(nt,(-ntelec("capital",nt))*ele(nt));                

mkt_pq(ff)..            
	(-sam("rent",ff)) =G= 
	(-sam("rent",ff))*x(ff)* 
	((theta_r_ff(ff)*pq(ff)**(1-esub_ff(ff)) +  (1-theta_r_ff(ff))*( theta_l_ff(ff)*pl  
	+ theta_k_ff(ff)*pk + theta_y_ff(ff)*py + theta_ele_ff(ff)*pele 
	+ sum(f$(not sameas(f,ff)), theta_f_ff(f,ff)*pf(f)))**(1-esub_ff(ff)))
	**(1/(1-esub_ff(ff)))/pq(ff))**esub_ff(ff);          

mkt_pn(n)..
	nrsupply(n) =G= sum(nt,(-ntelec(n,nt))*ele(nt)) + sum(xt,(-xtelec(n,xt))*ele(xt));;

mkt_pkx(xt)$kx(xt)..  
	kx(xt) =G= (kx(xt)/xtelec("ele",xt))*ele(xt);

mkt_plim(t)$limit(t).. 
	limit(t) =G= ele(t);

mkt_pcarb$(carblim and dd)..     
	carblim =G= sum(ff,carbon(ff)*x(ff));  

mkt_pc ..               
	c0*c =G= c0*w*(pw/pc)**esubl ; 

mkt_pg$dd ..
	g0*g =G= govt/pg ; 

mkt_pw..
	(c0+ld0)*w =G= ra/pw; 	

mkt_pls..
	(sam("labor","ra")/(1+tl0$dd)+ld0) =G= ld0*w*(pw/pls)**esubl 
	+ (sam("labor","ra")/(1+tl0$dd))*lsp; 	

*       Definition of income constraints
inc_ra..
	(-sam("capital","y")+ sum(ff,-sam("capital",ff)) 
	 + sum(xt,(-xtelec("capital",xt)) - kx(xt)))*pk
	+ sum(xt, pkx(xt)*kx(xt))
	+ (sam("labor","ra")/(1+tl0$dd)+ld0)*pls
	+ sum(ff,(-sam("rent",ff))*pq(ff))
	+ sum(n, nrsupply(n)*pn(n))
	+ sum(t$limit(t), limit(t)*plim(t))   
	- sum(t$quota(t), pele*ele(t)*psi) 
	- (pc*phi_ls)$dd   
	=G=  ra;

inc_govt$dd..   
	(carblim*pcarb)$dd + pc*phi_ls 
	+ (-sam("y","ra")/(1+tc0$dd))*c
	*( (pc/(py*(1+(tc0*phi_tc)$dd)))*(1+tc0$dd))**esubc*py*tc0*phi_tc     
	+ (sam("labor","ra")/(1+tl0$dd))*lsp*pls*tl0*phi_tl
	=G= govt;

*       Endogenous subsidy to assure renewables quota
eqy_psi$card(quota)..   
	sum(t$res(t), ele(t)) =G= t_shr*sum(t, ele(t));

*       Endogenous equal yield constraints
eqy_phi_ls$dd..             
	g =G= 1;

eqy_phi_tc$dd..             
	g =G= 1;          

eqy_phi_tl$dd..             
	g =G= 1;          


*       Define MCP model
model mcp_static / zprf_y.y, zprf_ele.ele, zprf_x.x, zprf_c.c, zprf_g.g, zprf_w.w, zprf_lsp.lsp,             
		   mkt_py.py, mkt_pele.pele, mkt_pf.pf, mkt_pl.pl, mkt_pls.pls, mkt_pk.pk, mkt_pq.pq, 
		   mkt_pn.pn, mkt_pcarb.pcarb, mkt_pkx.pkx, mkt_plim.plim, mkt_pc.pc, mkt_pg.pg, mkt_pw.pw, 
		   inc_ra.ra,inc_govt.govt,  
		   eqy_psi.psi, eqy_phi_ls.phi_ls, eqy_phi_tc.phi_tc, eqy_phi_tl.phi_tl  
                /;

*       Benchmark initialization

*       In the base year new-vintage technologies are inactive
*       and the prices of backstop natural resources are zero.
*       Extant technologies with capacity limits are assumed to
*       operate at the upper bound with a zero shadow value in the
*       base year.

ele.l(nt)  = 0; 
pn.l(n)    = 0;
plim.l(xt) = 0;
pkx.l(xt)  = 1;

ele.l(xt)  = xtelec("ele",xt);

*       Initialize activities and prices
y.l = 1; ele.l(xt)= xtelec("ele",xt);  x.l(ff) = 1; c.l = 1; w.l = 1; lsp.l = 1; 
py.l = 1; pele.l = 1; pf.l(ff) = 1; pl.l = 1; pls.l = 1; pk.l = 1; pq.l(ff) = 1;                                 
pkx.l(t)$kx(t) = 1; plim.l(t) = 0; pw.l = 1; pn.l(n) = 0; pc.l = 1; 

*       Install lower bounds on prices to avoid divison by zero in MCP formulation
py.lo = 1e-6; pele.lo = 1e-6; pf.lo(ff) = 1e-6; pl.lo = 1e-6; pk.lo = 1e-6; 
pq.lo(ff) = 1e-6; pkx.lo(t)$kx(t) = 1e-6; pc.lo = 1e-6; pw.lo = 1e-6; pls.lo = 1e-6;                                     

*       Assign inital values to further variables
phi_tc.fx = 1; phi_ls.fx = 0; phi_tl.fx = 1; g.fx = 0; pg.fx = 0; govt.fx = 0; pcarb.fx = 0;
psi.fx$(not card(quota)) = 0; plim.fx(t)$(not limit(t)) = 0; pkx.fx(t)$(not kx(t)) = 0;

*	Fix wage as the numeraire
pl.fx	= 1;

ra.l =  (-sam("capital","y")+ sum(ff,-sam("capital",ff)) 
	+ sum(xt,(-xtelec("capital",xt)) - kx(xt)))*pk.l
	+ sum(xt, pkx.l(xt)*kx(xt))
	+ (sam("labor","ra")/(1+tl0$dd)+ld0)*pls.l
	+ sum(ff,(-sam("rent",ff))*pq.l(ff))
	+ sum(n, nrsupply(n)*pn.l(n))
	+ sum(t$limit(t), limit(t)*plim.l(t))   
	- sum(t$quota(t), pele.l*ele.l(t)*psi.l) 
	- (pc.l*phi_ls.l)$dd;   

govt.l$dd = (carblim*pcarb.l)$dd + pc.l*phi_ls.l 
	    + (-sam("y","ra")/(1+tc0$dd))*c.l
	       *( (pc.l/(py.l*(1+(tc0*phi_tc.l)$dd)))*(1+tc0$dd))**esubc*py.l*tc0*phi_tc.l     
	     + (sam("labor","ra")/(1+tl0$dd))*pls.l*tl0*phi_tl.l;

mcp_static.iterlim = 0;
solve mcp_static using mcp;

display "The precision of the benchmark dataset is:", mcp_static.objval;
abort$(ABS(mcp_static.objval) gt 1e-4)"MCP model does not calibrate"; 

*	Relax iteration limit for counterfactual policy analysis
mcp_static.iterlim = 10000;


*===========================================================================
*	Static analysis of policy scenarios (as laid out in the paper)
*
*	Scenario 1:	Gradual nuclear phase-out
*	Scenario 2:	Target quota for renewables (green quota)
*	Scenario 3:	Carbon taxation (environmental tax reform)


*	Define report parameters
parameter
	ev	 Equivalent variation in income
	supply	 Electricity supply by technology
	carbtax	 Carbon permit price
	subsidy	 Subsidy rate on electricity from renewables;		

*---------  Scenario 1: Gradual nuclear phase-out --------------------------

set	nsc  Nuclear phase scenarios / 0, 25, 50, 75, 100/;


*	Sub-scenario: No supply limits on fossil fuel based power generation
loop(nsc,

*	Assign available capacity for nuclear power 
	limit("nuclear") = (1 - (ord(nsc)-1)/(card(nsc)-1))*xtelec("ele","nuclear");
*	If nuclear capacity is set to zero, assure complete nuclear phase out	
	if ((not limit("nuclear")),
	   ele.fx("nuclear") = 0;
	   );
	solve mcp_static using mcp;
	supply(nsc,t)	    = ele.l(t);
	ev(nsc,"fossil")    = round(100 * (w.l-1), 3) + eps;
);

display ev, supply;

*	Sub-scenario: Base-year limits on fossil fuel based power generation
ele.up("nuclear")=+inf;
limit("coal") = xtelec("ele","coal");
limit("gas")  = xtelec("ele","gas");
limit("oil")  = xtelec("ele","oil");

loop(nsc,

*	Assign available capacity for nuclear power 
	limit("nuclear") = (1 - (ord(nsc)-1)/(card(nsc)-1))*xtelec("ele","nuclear");

*	If nuclear capacity is set to zero, assure complete nuclear phase out	
	if ((not limit("nuclear")),  ele.fx("nuclear") = 0;  );

	solve mcp_static using mcp;
	supply(nsc,t)	    = ele.l(t);
	ev(nsc,"nofossil")      = 100 * (w.l-1) + eps;
);

display ev, supply;

*	See if GNUPLOT is installed:

$if not exist '%gams.sysdir%wgnuplot.exe' $exit
$if not exist '%gams.ldir%plot.gms'     $exit

*	If GNUPLOT is installed  -- generate figures at the terminal:

$setglobal labels nsc
$setglobal gp_opt0 "set data style linespoints"
$setglobal gp_opt1 "set key below"

$setglobal gp_opt2 "set title 'Technology shifts'"
$setglobal gp_opt3 "set xlabel 'Nuclear capacity reduction (% vis--vis BaU)'"
$setglobal gp_opt4 "set ylabel 'Activity level of technologies'"
$libinclude plot supply
supply(nsc,t) = 0; 

$setglobal gp_opt2 "set title 'Welfare changes'"
$setglobal gp_opt3 "set ylabel 'Equivalent variation in income (%)'"
$libinclude plot ev
ev(nsc,"fossil") = 0; ev(nsc,"nofossil") = 0;

*	Re-initialize parameterization for subsequent scenarios
ele.up("nuclear")= +inf;
limit(t) = 1e+6 * elecgen("total");


*------------------------ Scenario 2: Green quotas -------------------------

*	Note:	We start from the benchmark share of renewables in power
*		generation and then increase this share iteratively by 2% 
*		in five steps (towards a total increase of 10%).

scalar	bmk_shr		Benchmark renewable share in electricity generation;
bmk_shr = sum(t$res(t), xtelec("ele",t))/sum(t, xtelec("ele",t));

set	qsc  Iterative steps for green quota scenario / 1*6 /;

quota(res) = yes; psi.lo = -inf; psi.up = +inf; psi.l = 0;  
y.l = 1; ele.l(xt)= xtelec("ele",xt);  x.l(ff) = 1; c.l = 1; w.l = 1; lsp.l = 1; 
py.l = 1; pele.l = 1; pf.l(ff) = 1; pl.l = 1; pls.l = 1; pk.l = 1; pq.l(ff) = 1;                                 
pkx.l(t)$kx(t) = 1; plim.l(t) = 0; pw.l = 1; pn.l(n) = 0; pc.l = 1; 

*	First calculate long-run impact with malleable capital of extant technologies
loop(qsc,

*	Assign target shares for renewables in electricity production
	t_shr  = min(1, (bmk_shr + 10/100* (ord(qsc)-1)/(card(qsc)-1)));

	solve mcp_static using mcp;

	supply(qsc,t)		= ele.l(t)  + eps;
	ev(qsc,"long run")	= 100 * (w.l-1) + eps;
	subsidy(qsc,"long run")	= 100*psi.l + eps;
);


*	Then calculate short-run impact with fixed capital of extant technologies
kx(xt) = -xtelec("capital",xt); pkx.up(xt)$kx(xt) = +inf; pkx.l(xt)$kx(xt) = 1;  
psi.lo = 0; psi.up = +inf; psi.l = 0;  
y.l = 1; ele.l(xt)= xtelec("ele",xt);  x.l(ff) = 1; c.l = 1; w.l = 1; lsp.l = 1; 
py.l = 1; pele.l = 1; pf.l(ff) = 1; pl.l = 1; pls.l = 1; pk.l = 1; pq.l(ff) = 1;                                 
pkx.l(t)$kx(t) = 1; plim.l(t) = 0; pw.l = 1; pn.l(n) = 0; pc.l = 1; 

loop(qsc,
*	Assign target shares for renewables in electricity production
	t_shr  = min(1, (bmk_shr + 10/100* (ord(qsc)-1)/(card(qsc)-1)));

	solve mcp_static using mcp;

	supply(qsc,t)		 = ele.l(t)  + eps;
	ev(qsc,"short run")      = 100 * (w.l-1) + eps;
	subsidy(qsc,"short run") = 100*psi.l + eps;
);

$setglobal labels qsc
$setglobal gp_opt2 "set title 'Welfare changes'"
$setglobal gp_opt3 "set xlabel 'Green quota in % of overall electricity supply'"
$setglobal gp_opt4 "set ylabel 'Equivalent variation in income (%)'"
$setglobal gp_opt5 "set xtics ('18' 1, '20' 2, '22' 3, '24' 4, '26' 5, '28' 6)"

$libinclude plot ev
ev(qsc,"long run") = 0; ev(qsc,"short run") = 0; 

$setglobal gp_opt2 "set title 'Implicit Subsidy'"
$Setglobal gp_opt3 "set ylabel 'Subsidy rate (% of electricity price)'"
$libinclude plot subsidy
subsidy(qsc,"long run")	= 0; subsidy(qsc,"short run") = 0;

*	Re-initialize parameterization for subsequent scenarios
quota(t) = no; kx(t) = 0; psi.fx$(not card(quota)) = 0; pkx.fx(t)$(not kx(t)) = 0;


*------------------ Scenario 3: Carbon taxation (Environmental tax reform) -------
dd  = 1; 
g.up = +inf; pg.up = +inf; govt.up = +inf; pcarb.up = +inf;

*	Set public consumption at 15% of private consumption (of Y) 
g0  = 0.15 *(-sam("Y","ra")); 
tc0 = g0/((-sam("Y","ra")) - g0);


*	Initially, we assume that lump-sum transfers are active
*	as the equal-yield instrument
phi_ls.lo = -inf; phi_ls.up = +inf; phi_ls.l = -tl0*sam("labor","ra")/(1+tl0); 
phi_tc.fx = 1; phi_tl.fx  = 1; 


*	Benchmark replication check for the model with public good extension
*	Initialize activities and prices

y.l = 1; ele.l(xt)= xtelec("ele",xt);  ele.l(nt) = 0; x.l(ff) = 1; c.l = 1; g.l = 1;	
py.l = 1; pele.l = 1; pf.l(ff) = 1; pl.l = 1; pk.l = 1; pq.l(ff) = 1;					
pkx.l(t) = 1; plim.l(t)$limit(t) = 0; pc.l = 1; pn.l(n) = 0; 
pcarb.l = 0; pg.l = 1; lsp.l = 1; pls.l = 1; w.l = 1; pw.l = 1;

ra.l =  (-sam("capital","y")+ sum(ff,-sam("capital",ff)) 
	+ sum(xt,(-xtelec("capital",xt)) - kx(xt)))*pk.l
	+ sum(xt, pkx.l(xt)*kx(xt))
	+ (sam("labor","ra")/(1+tl0$dd)+ld0)*pls.l
	+ sum(ff,(-sam("rent",ff))*pq.l(ff))
	+ sum(n, nrsupply(n)*pn.l(n))
	+ sum(t$limit(t), limit(t)*plim.l(t))   
	- sum(t$quota(t), pele.l*ele.l(t)*psi.l) 
	- (pc.l*phi_ls.l)$dd;   

govt.l$dd = (carblim*pcarb.l)$dd + pc.l*phi_ls.l 
	    + (-sam("y","ra")/(1+tc0$dd))*c.l
	       *( (pc.l/(py.l*(1+(tc0*phi_tc.l)$dd)))*(1+tc0$dd))**esubc*py.l*tc0*phi_tc.l     
	     + (sam("labor","ra")/(1+tl0$dd))*pls.l*tl0*phi_tl.l;

mcp_static.iterlim = 0;

display "The precision of the re-specified benchmark dataset is:", mcp_static.objval;
abort$(ABS(mcp_static.objval) gt 1e-4)"MCP model does not calibrate"; 

*	Relax iteration limit
mcp_static.iterlim = 4000;

*	Specification of carbon tax scenarios based on exogenous emission reduction targets
set	csc  Carbon abatement scenarios scenarios / 0, 5, 10, 15, 20/;

loop(csc,
*	Assign carbon emission limit
	carblim = (1 - 0.2*(ord(csc)-1)/(card(csc)-1))*sum(ff, carbon(ff));

*	Activate lump-sum transfer as recycling instrument
	phi_ls.l  = 0; phi_ls.lo = -inf; phi_ls.up = +inf; phi_ls.l = -tl0*sam("labor","ra")/(1+tl0); 
	phi_tc.fx = 1; phi_tl.fx = 1;
	solve mcp_static using mcp;
	ev(csc,"ls") 	  =  round(100 * (w.l-1), 3) + eps;
	carbtax(csc,"ls") = round((1/multi)*pcarb.l/pc.l, 0) + eps;

*	Activate consumption tax as recycling instrument
	phi_tc.l = 1; phi_tc.lo = -0.99;  phi_tc.up = +inf; phi_ls.fx = -tl0*sam("labor","ra")/(1+tl0); phi_tl.fx = 1;
	solve mcp_static using mcp;
	ev(csc,"tc")	  = round(100 * (w.l-1), 3) + eps;
	carbtax(csc,"tc") = round((1/multi)*pcarb.l/pc.l, 0) + eps;

*	Activate wage tax as recycling instrument
	phi_tl.l = 1; phi_tl.lo = -0.99; phi_tl.up = +inf; phi_tc.fx = 1; phi_ls.fx = -tl0*sam("labor","ra")/(1+tl0);
	solve mcp_static using mcp;
	ev(csc,"tl") 	  = round(100 * (w.l-1), 3) + eps;
	carbtax(csc,"tl") = round((1/multi)*pcarb.l/pc.l, 0) + eps;
);
display ev, carbtax;

$setglobal labels csc
$setglobal gp_opt2 "set title 'Welfare changes'"
$setglobal gp_opt3 "set xlabel 'Carbon emission reduction (in % vis--vis base year)'"

$setglobal gp_opt4 "set ylabel 'Equivalent variation in income (%)'"
$libinclude plot ev
ev(csc,"tl") = 0; ev(csc,"tc") = 0; ev(csc,"ls") = 0;

$setglobal gp_opt2 "set title 'Carbon Tax Rate'"
$setglobal gp_opt4 "set ylabel 'Carbon tax in $ per ton of C'"
$libinclude plot carbtax
carbtax(csc,"tl") = 0; carbtax(csc,"tc") = 0; carbtax(csc,"ls") = 0;

